import datetime
import json
import time

import paho.mqtt.client as mqtt

# Statistics counter
msgs_counter = 0
lbs_counter = 0
dt = time.mktime(datetime.datetime.now().timetuple())
qos = 0

# Statistics dump interval
interval = 10

# MQTT connection credentials
mqtt_host = "mqtt.flespi.io"
mqtt_port = 1883
mqtt_token = "XXXXXXX"
mqtt_subscribe_topic = "debug_stream/#"
mqtt_lbs_topic = "hw/lbs"


def on_connect(client, userdata, flags, rc):
    """The callback for when the client receives a CONNACK response from
    the server

    """
    print("Connected to ", mqtt_host)

    # Subscribing in on_connect() means that if we lose the connection and
    # reconnect then subscriptions will be renewed.
    client.subscribe(mqtt_subscribe_topic)


def on_message(client, userdata, msg):
    """The callback for when a PUBLISH message is received from the server

    """
    global msgs_counter
    global dt
    global lbs_counter

    try:
        data = json.loads(msg.payload.decode('utf-8'))

        msgs_counter += 1
        # Process message
        if 'id' in data:
            # split message to fields
            fields = data['msg']
            pos = fields['pos']
            p = fields['p']

            # Check pos and lbs params in message
            if p and pos and 'mcc' in p and 'mnc' in p and 'lac' in p \
                    and 'cell_id' in p:
                lbs_counter += 1
                # send lbs message to specified topic
                lbs_msg = json.dumps(dict(pos=pos, params=p))
                client.publish(mqtt_lbs_topic, bytearray(lbs_msg.encode('utf-8')),
                               qos=qos
                )

    except Exception as e:
        pass

    # Dump and reset statistics counters
    ct = time.mktime(datetime.datetime.now().timetuple())
    if ct - dt > interval:
        print(
            "{}: {} msgs/sec, lbs: {} msgs/sec".format(datetime.datetime.now(),
                                                       msgs_counter / interval,
                                                       lbs_counter / interval)
        )

        dt = ct
        msgs_counter = 0
        lbs_counter = 0

# Create MQTT client
client = mqtt.Client()
client.on_connect = on_connect
client.on_message = on_message

client.username_pw_set(mqtt_token)
client.connect(mqtt_host, mqtt_port, 60)

# Blocking call that processes network traffic, dispatches callbacks and
# handles reconnecting.
# Other loop*() functions are available that give a threaded interface and a
# manual interface.
client.loop_forever()
